/*
 * barrowman.c -- compute C_p of a rocket using barrowmans method
 *
 * (c) 2005 Dr. Andreas Mueller, Beratung und Entwicklung
 * $Id$
 */
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <global.h>
#include <string.h>
#include <nosecone.h>
#include <cylinder.h>
#include <transition.h>
#include <finsection.h>
#include <usage.h>

/*
 * readargs -- read command line and create all components of the rocket while
 *             doing so
 */
static int	readargs(int argc, char *argv[], int i) {
	int	rc = 0;
	do {
		if (debug)
			fprintf(stderr, "%s:%d: interpreting argument %s\n",
				__FILE__, __LINE__, argv[i]);
		if (0 == strncmp(argv[i], "nose", 4)) {
			rc = nosecone_read(argc, argv, ++i);
		} else
		if (0 == strncmp(argv[i], "cyli", 4)) {
			rc = cylinder_read(argc, argv, ++i);
		} else
		if (0 == strncmp(argv[i], "tran", 4)) {
			rc = transition_read(argc, argv, ++i);
		} else
		if (0 == strncmp(argv[i], "fins", 4)) {
			rc = finsection_read(argc, argv, ++i);
		} else {
			fprintf(stderr, "unknown section type: %s\n", argv[i]);
			return -1;
		}
		if (rc < 0) {
			fprintf(stderr, "failed to construct new section %s\n",
				argv[i - 1]);
			return rc;
		}
		if (debug)
			fprintf(stderr, "%s:%d: construction of %s section "
				"used %d arguments\n",
				__FILE__, __LINE__, argv[i - 1], rc);
		i += rc;
	} while (i < argc);
	return 0;
}

int	main(int argc, char *argv[]) {
	int	c;

	/* read command line options */
	while (EOF != (c = getopt(argc, argv, "vdh?")))
		switch (c) {
		case 'd':
			debug++;
			break;
		case 'v':
			verbose = 1;
			break;
		case '?':
		case 'h':
			usage(argv[0]);
			exit(EXIT_SUCCESS);
			break;
		}

	/* now read all the other command line arguments */
	if (readargs(argc, argv, optind) < 0) {
		fprintf(stderr, "rocket construction failed\n");
		exit(EXIT_FAILURE);
	}

	/* the result of this is that the barrowman computation has been
	   completed, so we only need to display it */
	section_compute_all();
	if (verbose)
		section_print_all(stdout);
	printf("C_n = %10.3f\n", section_Cn());
	printf("X_n = %10.3f\n", section_Xn());

	/* that's it */
	exit(EXIT_SUCCESS);
}
