/*
 * cylinder.c -- compute barrowman for a cylinder
 *
 * (c) 2005 Dr. Andreas Mueller, Beratung und Entwicklung
 * $Id$
 */
#include <cylinder.h>
#include <string.h>
#include <stdio.h>

#define	PARAMS_LENGTH	1
param_t	cylinder_params[PARAMS_LENGTH] = {
	{ "length:", 0., 0 }
};

int	cylinder_read(int argc, char *argv[], int i) {
	param_t	*p;
	int	j = i;
	section_t	*cylinder = section_add(cylinder_new());
	if (debug)
		fprintf(stderr, "%s:%d: reading cylinder\n",
			__FILE__, __LINE__);
	while (NULL != (p = param_get(argv[j], cylinder_params,
		PARAMS_LENGTH))) {
		if (debug)
			fprintf(stderr, "%s:%d: found parameter %s(%d) = %f\n",
				__FILE__, __LINE__,
				cylinder_params[p->offset].name,
				p->offset, p->value);
		switch (p->offset) {
		case 0:	cylinder->length = p->value;
			break;
		}
		j++;
	}
	return j - i;
}

static int	cylinder_compute(section_t *s, double d) {
	if (debug)
		fprintf(stderr, "%s:%d: computing Cn/deltaX for a cylinder\n",
			__FILE__, __LINE__);
	s->deltaX = 0.;
	s->Cn = 0.;
	return 0;
}

static void	cylinder_print(section_t *s, FILE *f, int *width) {
	int	i = *width, o = outerspace - *width;
	fprintf(f, "%*.*s|%*.*s|%*.*s", o, o, "", 2 * i, 2 * i, "", o, o, "");
	fprintf(f, " length:%9.3f\n", s->length);
}

section_t	*cylinder_new() {
	section_t	*result = section_new();
	result->type = SECTION_CYLINDER;
	result->compute = cylinder_compute;
	result->print = cylinder_print;
	if (debug)
		fprintf(stderr, "%s:%d: new cylinder created\n",
			__FILE__, __LINE__);
	return result;
}
