/*
 * finsection.c -- compute barrowman for a finsection
 *
 * (c) 2005 Dr. Andreas Mueller, Beratung und Entwicklung
 * $Id$
 */
#include <finsection.h>
#include <string.h>
#include <math.h>
#include <stdio.h>

#define	PARAMS_LENGTH	7
param_t	finsection_params[PARAMS_LENGTH] = {
	{ "length:", 0., 0 },
	{ "a:", 0., 1 },
	{ "b:", 0., 2 },
	{ "m:", 0., 3 },
	{ "diameter:", 0., 4 },
	{ "s:", 0., 5 },
	{ "n:", 0., 6 }
};

int	finsection_read(int argc, char *argv[], int i) {
	int	j = i;
	section_t	*finsection = section_add(finsection_new());
	param_t	*p;
	while (NULL != (p = param_get(argv[j], finsection_params,
		PARAMS_LENGTH))) {
		if (debug)
			fprintf(stderr, "%s:%d: found parameter %s(%d) = %f\n",
				__FILE__, __LINE__,
				finsection_params[p->offset].name, p->offset,
				p->value);
		switch (p->offset) {
		case 0:	finsection->length = p->value;
			break;
		case 1:	finsection->detail.finsection.a = p->value;
			break;
		case 2:	finsection->detail.finsection.b = p->value;
			break;
		case 3:	finsection->detail.finsection.m = p->value;
			break;
		case 4:	finsection->detail.finsection.r = p->value;
			break;
		case 5:	finsection->detail.finsection.s = p->value;
			break;
		case 6:	finsection->detail.finsection.n = p->value;
			break;
		}
		j++;
	}
	return j - i;
}

static int	finsection_compute(section_t *p, double d) {
	double	a, b, r, s, m, l, k;
	int	n;
	if (debug)
		fprintf(stderr, "%s:%d: computing fin section\n",
			__FILE__, __LINE__);
	a = p->detail.finsection.a;
	b = p->detail.finsection.b;
	m = p->detail.finsection.m;
	r = p->detail.finsection.r;
	s = p->detail.finsection.s;
	n = p->detail.finsection.n;
	l = sqrt(sqr(m + 0.5 * (b - a)) - sqr(s - r));
	p->Cn = 4. * n * sqr(s/d)/(1 + sqrt(1 + sqr(2. * l/(a + b))));
	if (p->detail.finsection.n <= 4) {
		k = 1 + r/(s + r);
	} else {
		k = 1 + 0.5 * r/(s + r);
	}
	if (debug)
		fprintf(stderr, "%s:%d: fin interference factor: %f\n",
			__FILE__, __LINE__, k);
	p->Cn *= k;
	p->deltaX = m * (a + 2. * b)/(3. * (a + b))
		+ 0.16666 * (a + b - a * b/(a + b));
	if (debug)
		fprintf(stderr, "%s:%d: Cn = %f, deltaX = %f\n",
			__FILE__, __LINE__, p->Cn, p->deltaX);
	return 0;
}

static void	finsection_print(section_t *s, FILE *f, int *width) {
	int	i = *width, o = outerspace - *width;
	fprintf(f, "%*.*s#|%*.*s|#%*.*s", o - 1, o - 1, "", 2 * i, 2 * i, "", o - 1, o - 1, "");
	fprintf(f, " length:%9.3f\n", s->length);
}

section_t	*finsection_new() {
	section_t	*result = section_new();
	result->type = SECTION_FINSECTION;
	result->compute = finsection_compute;
	result->print = finsection_print;
	if (debug)
		fprintf(stderr, "%s:%d: new finsection added\n",
			__FILE__, __LINE__);
	return result;
}
