/* 
 * global.h -- global objects
 *
 * (c) 2005 Dr. Andreas Mueller, Beratung und Entwicklung
 */
#ifndef _GLOBAL_H
#define _GLOBAL_H

#include <stdio.h>

extern int	debug;
extern int	verbose;
extern int	outerspace;

typedef enum nosetype_e  {
	NOSE_CONE = 0,
	NOSE_OGIVE = 1,
	NOSE_PARABOLIC = 2
} nosetype_t;

typedef struct nosecone_s {
	nosetype_t	type;
	double	radius;
} nosecone_t;

typedef struct transition_s {
	double	radius1, radius2;
} transition_t;

typedef struct cylinder_s {
} cylinder_t;

typedef struct finsection_s {
	double	r, s, a, b, m;
	int	n;
} finsection_t;

typedef enum sectiontype_e {
	SECTION_NOSECONE = 0,
	SECTION_CYLINDER = 1,
	SECTION_TRANSITION = 2,
	SECTION_FINSECTION = 3
} sectiontype_t;

typedef struct section_s {
	double	length;
	double	deltaX;
	double	Cn;
	sectiontype_t	type;
	union {
		nosecone_t	nosecone;
		cylinder_t	cylinder;
		transition_t	transition;
		finsection_t	finsection;
	}	detail;
	struct section_s	*next;
	int	(*compute)(struct section_s *, double);
	void	(*print)(struct section_s *, FILE *, int *);
} section_t;

extern section_t	*rocket;

extern section_t	*section_first();
extern section_t	*section_next(section_t *current);
extern section_t	*section_new();
extern section_t	*section_add(section_t *add);
extern int	section_compute(section_t *s, double d);
extern int	section_compute_all();
extern double	section_Cn();
extern double	section_Xn();
extern void	section_print(section_t *s, FILE *f, int *width);
extern void	section_print_all(FILE *f);

/*
 * auxiliary data types and parameters for command line parsing
 */
typedef struct param_s {
	char	*name;
	double	value;
	int	offset;
} param_t;

param_t	*param_get(char *arg, param_t *params, int n);

/*
 * some auxiliary functions
 */
extern double	sqr(double x);

#endif /* _GLOBAL_H */
