/*
 * nosecone.c -- compute barrowman for a nosecone
 *
 * (c) 2005 Dr. Andreas Mueller, Beratung und Entwicklung
 * $Id$
 */
#include <nosecone.h>
#include <string.h>
#include <stdio.h>

#define	PARAMS_LENGTH	2
param_t	nosecone_params[PARAMS_LENGTH] = {
	{ "length:", 0., 0 },
	{ "diameter:", 0., 1 }
};

int	nosecone_read(int argc, char *argv[], int i) {
	int	j = i;
	param_t	*p;
	nosetype_t	type;
	section_t	*nose = section_add(nosecone_new());
	if (*argv[j] == 'c') {
		type = NOSE_CONE;
	} else if (*argv[j] == 'o') {
		type = NOSE_OGIVE;
	} else if (*argv[j] == 'p') {
		type = NOSE_PARABOLIC;
	} else {
		fprintf(stderr, "unknown nosecone shape: %s\n", argv[j]);
		return -1;
	}
	if (debug)
		fprintf(stderr, "%s:%d: known nosecone type: %s\n",
			__FILE__, __LINE__,
			(type == NOSE_CONE) ? "cone" :
			((type == NOSE_OGIVE ) ? "ogive" : "parabolic"));
	nose->detail.nosecone.type = type;
			
	j++;
	while (NULL != (p = param_get(argv[j], nosecone_params,
		PARAMS_LENGTH))) {
		if (debug)
			fprintf(stderr, "%s:%d: found parameter %s(%d) = %f\n",
				__FILE__, __LINE__,
				nosecone_params[p->offset].name, p->offset,
				p->value);
		switch (p->offset) {
		case 0:	nose->length = p->value;
			break;
		case 1:	nose->detail.nosecone.radius = p->value/2.;
			break;
		}
		j++;
	}
	return j - i;
}

static int	nosecone_compute(section_t *s, double d) {
	s->Cn = 2.;
	switch (s->detail.nosecone.type) {
	case NOSE_CONE:
		s->deltaX = 2. * s->length / 3.;
		break;
	case NOSE_OGIVE:
		s->deltaX = 0.466 * s->length;
		break;
	case NOSE_PARABOLIC:
		s->deltaX = 0.5 * s->length;
		break;
	}
	if (debug)
		fprintf(stderr, "%s:%d: compute nosecone Cn = %f, deltaX = %f\n",
			__FILE__, __LINE__, s->Cn, s->deltaX);
	return 0;
}

static void	nosecone_print(section_t *s, FILE *f, int *width) {
	int	i = *width, o = outerspace - i;
	nosetype_t	t = s->detail.nosecone.type;
	fprintf(f, "%*.*s%c%*.*s%c%*.*s",
		o, o, "",
		(t == NOSE_CONE) ? '/' : ((t == NOSE_OGIVE) ? '(' : '+'),
		2 * i, 2 * i, "",
		(t == NOSE_CONE) ? '\\' : ((t == NOSE_OGIVE) ? ')' : '+'),
		o, o, "");
	(*width)++;
	fprintf(f, " length:%9.3f\n", s->length);
}

section_t	*nosecone_new() {
	section_t	*result = section_new();
	result->type = SECTION_NOSECONE;
	result->compute = nosecone_compute;
	result->print = nosecone_print;
	if (debug)
		fprintf(stderr, "%s:%d: new nosecone added\n",
			__FILE__, __LINE__);
	return result;
}
