/*
 * transition.c -- compute barrowman for a transition
 *
 * (c) 2005 Dr. Andreas Mueller, Beratung und Entwicklung
 * $Id$
 */
#include <transition.h>
#include <string.h>
#include <stdio.h>

#define	PARAMS_LENGTH	3
param_t	transition_params[PARAMS_LENGTH] = {
	{ "length:", 0., 0 },
	{ "diameter1:", 0., 1 },
	{ "diameter2:", 0., 2 }
};

int	transition_read(int argc, char *argv[], int i) {
	int	j = i;
	section_t	*transition = section_add(transition_new());
	param_t	*p;
	while (NULL != (p = param_get(argv[j], transition_params,
		PARAMS_LENGTH))) {
		if (debug)
			fprintf(stderr, "%s:%d: found parameter %s(%d) = %f\n",
				__FILE__, __LINE__,
				transition_params[p->offset].name, p->offset,
				p->value);
		switch (p->offset) {
		case 0:	transition->length = p->value;
			break;
		case 1:	transition->detail.transition.radius1 = p->value/2.;
			break;
		case 2:	transition->detail.transition.radius2 = p->value/2.;
			break;
		}
		j++;
	}
	return j - i;
}

static int	transition_compute(section_t *s, double d) {
	double	d1 = 2. * s->detail.transition.radius1;
	double	d2 = 2. * s->detail.transition.radius2;
	s->Cn = 2. * (sqr(d2) - sqr(d1))/sqr(d);
	s->deltaX = s->length * (1 + (1 - d1/d2)/(1 - sqr(d1/d2))) / 3.;
	if (debug)
		fprintf(stderr, "%s:%d: computing transition Cn=%f deltaX=%f\n",
			__FILE__, __LINE__, s->Cn, s->deltaX);
	return 0;
}

void	transition_print(section_t *s, FILE *f, int *width) {
	int	i, o;
	int	increase
		= (s->detail.transition.radius1 > s->detail.transition.radius2) ? 0 : 1;
	if (!increase)
		(*width)--;
	i = *width;
	o = outerspace - i;
	fprintf(f, "%*.*s%c%*.*s%c%*.*s",
		o, o, "",
		(increase) ? '/' : '\\',
		2 * i, 2 * i, "",
		(increase) ? '\\' : '/',
		o, o, "");
	if (increase)
		(*width)++;
	fprintf(f, " length:%9.3f\n", s->length);
}

section_t	*transition_new() {
	section_t	*result = section_new();
	result->type = SECTION_TRANSITION;
	result->compute = transition_compute;
	result->print = transition_print;
	if (debug)
		fprintf(stderr, "%s:%d: adding new transition\n",
			__FILE__, __LINE__);
	return result;
}
